@@ -126,6 +126,7 @@ proc Folder_Sort {} {
     global exmh
 
     if {[Ftoc_Changes "Sort"] == 0} then {
+	Background_Wait
 	Exmh_Status "Sorting folder..." blue
 	Msg_CheckPoint
 	Mh_Sort $exmh(folder)
@@ -138,6 +139,7 @@ proc Folder_Pack {} {
     global exmh
 
     if {[Ftoc_Changes "Pack"] == 0} then {
+	Background_Wait
 	Exmh_Status "Packing folder..." blue
 	Msg_CheckPoint
 	Mh_Pack $exmh(folder)
@@ -155,7 +157,8 @@ proc FolderCommit { rmmCommit moveCommit } {
     Ftoc_RangeUnHighlight
     Ftoc_Commit $rmmCommit $moveCommit
     Msg_ShowCurrent
-    Exmh_CheckPoint
+    Background_Wait	;# Let folder ops complete
+    Exmh_CheckPoint	;# before writing out cache
     Label_Folder $exmh(folder)
     # The following isn't strictly necessary, but it fixes up any
     # display bugs caused by races between exmh-bg FindUnseen and the UI
