@@ -20,9 +20,24 @@ proc Ftoc_Init {} {
     }
     set ftoc(mono) [string match mono* [tk colormodel .]]
     # Parameters to the Next button
-    Preferences_Resource ftoc(implied) impliedDirection 1
-    Preferences_Resource ftoc(nextGuard) nextGuard 0
-    Preferences_Resource ftoc(autoCommit) autoCommit 0
+    Preferences_Add "Scan Listing" \
+"These settings affect the behavior of Exmh as you move through the scan listing to view and mark messages.
+While the default for Auto Commit is OFF, I suggest you try it out.
+Messages are still temporarily marked, but the commit is done when you need it." {
+    {ftoc(implied) impliedDirection ON "Implied Direction"
+"If set, Exmh will remember your current direction,
+next or previous, and go that way after you mark a
+message for deletion or refiling."}
+    {ftoc(nextGuard) nextGuard OFF "Next Guard"
+"If set, Exmh will warn you that you are about to
+change folders when you hit Next.  This means you
+end up hitting Next twice to chain the to next
+folder with unseen messages."}
+    {ftoc(autoCommit) autoCommit OFF "Auto Commit"
+"If set, Exmh will invoke the Commit operation to
+commit deletions and refiles when it would otherwise
+just complain that such a commit is required."}
+    }
 }
 proc Ftoc_Reset { numMsgs msgid folder } {
     global ftoc exwin
@@ -67,7 +82,7 @@ proc Ftoc_Bindings { w } {
     bind $w <Shift-B1-Motion> {
 	FtocRangeExtendXY %x %y
     }
-    bind $w <ButtonRelease-1> {
+    bind $w <Any-ButtonRelease-1> {
 	FtocRangeEnd [lindex [split [%W index current] .] 0]
     }
     bind $w <Shift-ButtonRelease-1> {
@@ -103,6 +118,11 @@ proc FtocRangeAdd { line } {
 proc FtocRangeEnd { {line {}} {addcurrent false} } {
     # For end of button sweep
     global ftoc exwin
+    set ftoc(extend) 0
+    if ![info exists ftoc(pickend)] {
+	# Spurious button-release event
+	return
+    }
     if {$line != {}} {
 	FtocRangeExtend $line
     }
@@ -116,6 +136,7 @@ proc FtocRangeEnd { {line {}} {addcurrent false} } {
 	lappend lineset $line
     }
     FtocPickRange $lineset $addcurrent
+    unset ftoc(pickend)
 }
 proc Ftoc_PickMsgs { ids } {
     # For adding to the selection by message number
@@ -133,6 +154,9 @@ proc Ftoc_PickMsgs { ids } {
 proc FtocRangeExtendXY { x y } {
     global ftoc exwin widgetText
 
+    if ![info exists ftoc(extend)] {
+	return
+    }
     set active $ftoc(extend)
 
     set h [winfo height $exwin(ftext)]
@@ -174,6 +198,9 @@ proc FtocSelExtend {} {
 }
 proc FtocRangeExtend { line } {
     global ftoc
+    if ![info exists ftoc(pickend)] {
+	return
+    }
     if {$line <= 0} {
 	set line 1
     }
@@ -380,7 +407,18 @@ proc Ftoc_BindRight { cmd } {
 proc Ftoc_FindMsg { msgid {line {}} } {
     global ftoc
     if {$line != {}} {
-	return $line
+	switch -glob -- $line {
+	    first  {return 1}
+	    last   {return $ftoc(numMsgs)}
+	    [0-9]* {
+		if {$line > $ftoc(numMsgs)} {
+		    return $ftoc(numMsgs)
+		} else {
+		    return $line
+		}
+	    }
+	    default {return {}}
+	}
     }
     if {$msgid == {}} {
 	return {}
@@ -478,7 +516,6 @@ proc Ftoc_ClearCurrent {} {
     set ftoc(pickone) 1
     set ftoc(lineset) {}
 
-    $exwin(ftext) configure -state normal
     if {$ftoc(curLine) != {}} {
 	$exwin(ftext) tag remove current $ftoc(curLine).0 $ftoc(curLine).end
 	Ftoc_RescanLine $ftoc(curLine)
@@ -500,7 +537,6 @@ proc Ftoc_Change { msgid line {show show} } {
 	$exwin(ftext) yview -pickplace [expr $ftoc(curLine)-1]
 	set ok 1
     }
-    $exwin(ftext) configure -state disabled
     return $ok
 }
 proc FtocHackCurrentHighlight {} {
@@ -548,21 +584,31 @@ proc Ftoc_RescanLine { ix {plus none} } {
     global exmh exwin
     if [catch {
 	set text [$exwin(ftext) get ${ix}.0 ${ix}.end]
-	if {$plus == "none"} {
-	    # Replace + (current marker) with blank
-	    set ok [regsub {( *[0-9]+)(\+)} $text {\1 } newtext]
-	} else {
-	    # Stick a + after the number, if needed
-	    if ![regexp {( *)([0-9]+)(\+)} $text] {
-		set ok [regsub {( *[0-9]+)( )} $text {\1+} newtext]
-	    } else {
-		set ok 0
+	set ok 0
+	case $plus {
+	    "none" {
+		# Replace + (current marker) with blank
+		set ok [regsub {( *[0-9]+)(\+)} $text {\1 } newtext]
+	    }
+	    "+" {
+		# Stick a + after the number, if needed
+		if ![regexp {( *)([0-9]+)(\+)} $text] {
+		    set ok [regsub {( *[0-9]+)( )} $text {\1+} newtext]
+		}
+	    }
+	    "dash" {
+		# Stick a - after the number, if needed
+		if ![regexp {( *)([0-9]+).-} $text] {
+		    set ok [regsub {( *[0-9]+.)(.)} $text {\1-} newtext]
+		}
 	    }
 	}
 	if {$ok} {
 	    set tags [$exwin(ftext) tag names ${ix}.0]
+	    $exwin(ftext) configure -state normal
 	    $exwin(ftext) delete ${ix}.0 ${ix}.end
 	    $exwin(ftext) insert ${ix}.0 $newtext
+	    $exwin(ftext) configure -state disabled
 	    foreach tag $tags {
 		$exwin(ftext) tag add $tag ${ix}.0 ${ix}.end
 	    }
@@ -583,7 +629,7 @@ proc Ftoc_Next { show {implied no} } {
     } else {
 	set ftoc(direction) "next"
     }
-    Exmh_Debug Ftoc_Next
+    Exmh_Debug Ftoc_Next curline=$ftoc(curLine)
     if {$ftoc(curLine) == {}} {
 	# No current; go for the next unseen.
 	if [Msg_ShowUnseen] {
@@ -689,7 +735,7 @@ proc Ftoc_Changes {type {allowAuto 1} } {
     global ftoc
 
     if {$ftoc(changed) != 0} then {
-	if {$allowAuto && $ftoc(autoCommit)} {
+	if {("$allowAuto" == "1") && $ftoc(autoCommit)} {
 	    Folder_Commit
 	    return 0
 	}
@@ -798,7 +844,7 @@ proc Ftoc_Commit { rmmCommit moveCommit } {
     if {$l > 0} {
 	WidgetTextYview $exwin(ftext) $l
     }
-    if {! [Ftoc_Changes {}]} {
+    if {! [Ftoc_Changes {} noautocommit]} {
 	Exmh_Status "ok" blue
     }
 }
@@ -825,8 +871,12 @@ proc FtocCommit {tagname commitProc} {
 	    # Batch up deletes
 	    lappend delmsgs $msgid
 	} else {
-	    # Build up a list of moved messages, maintaining order
-	    set movemsgs($F) [concat $movemsgs($F) " " $msgid]
+	    # Build up a list of moved messages
+	    # Note that the original order of the messages is maintained,
+	    # (We are going from bottom to top thru the display.)
+	    # The scan lines are reversed, which is handled by Scan_Move.
+#	    set movemsgs($F) [concat $movemsgs($F) " " $msgid]
+	    set movemsgs($F) [concat $msgid " " $movemsgs($F)]
 	    lappend movescan($F) [$exwin(ftext) get $c0 "$ce + 1 chars"]
 	}
 	Msg_UnSeen $msgid	;# avoid MH mark bug
@@ -896,15 +946,20 @@ proc FtocMakeReversePairs { list } {
 }
 
 proc Ftoc_MoveFeedback { msgid line } {
-    global exwin
-    Exmh_Debug Ftoc_MoveFeedback $msgid $line
+    global exwin ftoc
+    set msg [Exmh_OldStatus]
+    if {[string compare $line "last"] == 0} {
+	set line $ftoc(numMsgs)
+    } elseif {[string compare $line first] == 0} {
+	set line 1
+    }
     foreach tag [$exwin(ftext) tag names $line.0] {
 	if [regexp {moved (.+)} $tag match folder] {
 	    Exmh_Status "$msgid => +$folder"
 	    return
 	}
     }
-    Exmh_Status ""
+    Exmh_Status $msg
 }
 proc Ftoc_FindNext {} {
     Find_It forw
