@@ -12,31 +12,22 @@
 
 proc Pick {} {
     global pick
-    if {[catch {wm state .pick} state] == 0} {
-	if {$state != "normal"} {
-	    catch {wm geometry .pick $pick(geometry)}
-	    wm deiconify .pick
-	} else {
-	    catch {raise .pick}
-	}
-	return
+    if [Exwin_Toplevel .pick "Pick Messages" Pick] {
+	set t .pick
+	set f .pick.but
+
+	.pick.but.quit configure -command {Exwin_Dismiss .pick nosize}
+	Widget_AddBut $f clear "Clear" { PickClear }
+	Widget_AddBut $f mark "Mark Seen" {PickMarkSeen}
+	Widget_AddBut $f pick "Pick" {Pick_It} {left padx 1}
+	set pick(addtosel) 0
+	Widget_CheckBut $f add "Add to Sel" pick(addtosel) {left padx 1}
+	set pick(project) 0
+	Widget_CheckBut $f project "New FTOC" pick(project) {left padx 1}
+    
+	set pick(panes) 0
+	PickButtons
     }
-    catch {destroy .pick}
-    set t [Widget_Toplevel .pick "Pick Messages" Pick]
-    catch {wm geometry .pick $pick(geometry)}
-
-    set f [Widget_Frame .pick but Menubar]
-    Widget_AddBut $f dismiss "Dismiss" { PickDismiss }
-    Widget_AddBut $f clear "Clear" { PickDestroy; Pick }
-    Widget_AddBut $f mark "Mark Seen" {PickMarkSeen}
-    Widget_AddBut $f pick "Pick" {Pick_It} {left padx 1}
-    set pick(addtosel) 0
-    Widget_CheckBut $f add "Add to Sel" pick(addtosel) {left padx 1}
-    set pick(project) 0
-    Widget_CheckBut $f project "New FTOC" pick(project) {left padx 1}
-
-    set pick(panes) 0
-    PickButtons
 }
 proc PickButtons {} {
     global pick
@@ -52,6 +43,7 @@ proc PickButtons {} {
     PickEntry $f cc Cc
     PickEntry $f before Before
     PickEntry $f after After
+    PickEntry $f search Search
     set pad [Widget_Frame $f pad Pad]
     $pad configure -height 10 -width 10
     PickEntry $f msgs Messages
@@ -80,27 +72,24 @@ proc PickEntry { frame name label } {
     set pick(entry,$pick(panes),$name) [Widget_Entry $f1 entry {left expand fill}]
     bind $pick(entry,$pick(panes),$name) <Return> {Pick_It}
 }
-proc PickSetGeom {} {
+
+proc PickClear {} {
     global pick
-    set pick(geometry) [string trimleft [wm geom .pick] x0123456789]
-}
-proc PickDismiss {} {
-    PickSetGeom
-    wm withdraw .pick
-}
-proc PickDestroy {} {
-    PickSetGeom
-    destroy .pick
+    for {set pane 1} {$pane <= $pick(panes)} {incr pane} {
+	destroy .pick.rim$pane
+    }
+    set pick(panes) 0
+    PickButtons
 }
 proc Pick_It {} {
     global pick exmh
-    set cmd [list exec pick +$exmh(folder)]
+    set cmd [list exec pick +$exmh(folder) -list]
     set inpane 0
     set hadpane 0
     set msgs {}
     for {set pane 1} {$pane <= $pick(panes)} {incr pane} {
 	set and 0
-	foreach field {subject from to cc before after} {
+	foreach field {subject from to cc before after search} {
 	    set text [$pick(entry,$pane,$field) get]
 	    if {$text != {}} {
 		if {$inpane != $pane} {
@@ -143,7 +132,9 @@ proc PickInner {cmd msgs} {
 	Ftoc_RangeUnHighlight
     }
     if {$pick(project)} {
-	Scan_ProjectSelection $pick(ids)
+	if {[Ftoc_Changes pick] == 0} {
+	    Scan_ProjectSelection $pick(ids)
+	}
     } else {
 	Ftoc_PickMsgs $pick(ids)
     }
