@@ -12,11 +12,9 @@
 # any specification.
 
 proc SeditHelp {} {
-    catch {destroy .sedithelp}
-    set t [Widget_Toplevel .sedithelp "Sedit help" Help]
-    Widget_Frame $t but Menubar {top fill}
-    Widget_AddBut $t.but quit "Dismiss" [list destroy $t]
-    Widget_Message $t msg -text \
+    if [Exwin_Toplevel .sedithelp "Sedit help" Help] {
+	set t .sedithelp
+	Widget_Message $t msg -text \
 "
 Sedit is a simple editor built into exmh.
 The editor works on draft messages that are kept in your
@@ -24,25 +22,29 @@ MH draft folder.  When you do Compose, Reply, or Forward,
 a new draft is created for you.
 
 Here is what the buttons do:
-Help - displays this window.
-Bind - displays a dialog box that lets you edit keybindings.
-    See the TK bind man page for keystroke syntax, if you need to.
-Whom - displays the list of recipients for the message.
+
 Abort - remove the draft message from the drafts folder.
     The editor window is removed.
+Bind - displays a dialog box that lets you edit keybindings.
+    See the TK bind man page for keystroke syntax, if you need to.
+Help - displays this window.
+Insert @ - Insert a copy of the current message (replies only)
+Send - post the message.
 Sign - insert your ~/.signature file at the end of the message.
 Sign... - A menu of ~/.signature* files if you have more than one.
-Send - post the message.
+Whom - displays the list of recipients for the message.
 
 More... - Bring up a menu with the following options:
 Keep on Send - if this button is highlighted, then Send and Save will not
     remove the editor window.  Otherwise, the window disappears afterwards.
-Format mail - this breaks long lines (> 80 characters) at word boundaries.
+Format mail - this breaks long lines (> 79 characters) at word boundaries.
     Note that the auto-wrapping done by the TK text widget does not insert
     newline characters.  This will be done by the editor when you save
     the message and the Format mail option is selected.
+Attempt mhn - run the message body through mhn before sending.  If you know
+    the # directives of mhn, you can use this quick hack to compose MIME
+    messages.  A GUI alternative will appear someday.
 Insert File... - Insert a file into the message.
-Insert @ - Insert a copy of the current message (replies only)
 Save - write out the editor buffer to disk.  This is useful if you want
     to re-edit the draft message later.
 Spell... - run UNIX spell over the message body.
@@ -51,6 +53,7 @@ Dismiss - remove the editor window.  You will be prompted if the message
     has not been saved or sent.  If you abort the message from the dialog,
     it gets deleted from the drafts folder.
 "
+    }
 }
 
 proc Sedit_Start { draft } {
@@ -59,26 +62,20 @@ proc Sedit_Start { draft } {
 	Sedit_Init
     }
     set id [file tail $draft]
-    if [catch {Widget_Toplevel .sedit$id $draft Sedit} err] {
+    if {[Exwin_Toplevel .sedit$id $draft Sedit] == 0} {
 	# Reuse existing window
 	set t $sedit($id,text)
+	set sedit($t,format) $sedit(formatDefault)
 	SeditMsg $t $draft
 	$t delete 1.0 end
-	catch {
-	    if [file readable "@"] {
-		Widget_AddBut .sedit$id.but repl "@" \
-		    [list SeditInsertFile $draft $t "@"]
-	    } else {
-		destroy .sedit$id.but.repl
-	    }
-	}
     } else {
-	set b [Widget_Frame .sedit$id but Menubar {top fill}]
+	set b .sedit${id}.but
 	set f [Widget_Frame .sedit$id f Frame {top expand fill}]
 	set t [Widget_Text $f $sedit(height) -cursor xterm -setgrid true]
-	SeditTextBindings $draft $t
+	set sedit($t,status) [Widget_Entry .sedit${id} status {top fill} -relief raised]
+	$b.quit configure -command [list SeditQuit $draft $t]
 
-	Widget_AddBut $b send "Send" [list SeditSend $draft $t]
+	Widget_AddBut $b send "Send" [list SeditSend $draft $t] {right padx 10}
 	if [catch {glob ~/.signature*} sigFiles1] {
 	    set sigFiles1 [glob ~]/.signature
 	}
@@ -100,9 +97,12 @@ proc Sedit_Start { draft } {
 	Widget_AddBut $b pref "Bind" [list Sedit_Pref]
 	Widget_AddBut $b help "Help" [list SeditHelp]
 
+	Widget_AddBut $b repl "Insert @" {}
 	set menub [Widget_AddMenuB $b text "Text..." {right padx 1}]
 	Widget_AddMenuItem $menub "Plain" \
 	    [list SeditMimeEnriched $draft $t x-plain]
+	Widget_AddMenuItem $menub "Fixed" \
+	    [list SeditMimeEnriched $draft $t fixed]
 	Widget_AddMenuItem $menub "Bold" \
 	    [list SeditMimeEnriched $draft $t bold]
 	Widget_AddMenuItem $menub "Italic" \
@@ -116,24 +116,34 @@ proc Sedit_Start { draft } {
 
 	set menub [Widget_AddMenuB $b file "More..." {right padx 1}]
 	set sedit($t,keep) 0
-	set sedit($t,format) 1
+	set sedit($t,format) $sedit(formatDefault)
+	set sedit($t,mhn) $sedit(mhnDefault)
 	Widget_CheckMenuItem $menub "Keep on send" { } sedit($t,keep)
 	Widget_CheckMenuItem $menub "Format mail" { } sedit($t,format)
-	if [file readable "@"] {
-	    Widget_AddMenuItem $menub "Insert @" [list SeditInsertFile $draft $t "@"]
-	}    
+	Widget_CheckMenuItem $menub "Attempt mhn" { } sedit($t,mhn)
 	Widget_AddMenuItem $menub "Insert File..." [list SeditInsertFileDialog $draft $t]
-	Widget_AddMenuItem $menub "Abort" [list SeditAbort $draft $t]
 	Widget_AddMenuItem $menub "Save" [list SeditSave $draft $t]
 	Widget_AddMenuItem $menub "Spell..." [list SeditSpell $draft $f $t]
 	Widget_AddMenuItem $menub "Find SEL" [list Sedit_Find $draft $t]
-	Widget_AddMenuItem $menub "Dismiss" [list SeditQuit $draft $t]
 
-	set sedit($t,status) [Widget_Entry $b status {left fill expand}]
+	Widget_AddBut $b abort "Abort" [list SeditAbort $draft $t] {left padx 3}
+
 	SeditMsg $t $draft
+
+	# Define a bunch of maps among things
 	set sedit($t,toplevel) .sedit$id
 	set sedit($id,text) $t
+	set sedit($t,id) $id
 	lappend sedit(allids) .sedit$id
+	set sedit(.sedit$id,draft) $draft
+	set sedit(.sedit$id,id) $id
+    }
+    SeditTextBindings $draft $t		;# set up sendMsg binding
+    if [file readable "@"] {
+	.sedit$id.but.repl configure -state normal -command \
+		[list SeditInsertFile $draft $t "@"]
+    } else {
+	.sedit$id.but.repl configure -state disabled
     }
     set sedit($t,sent) 0
     set sedit($t,dirty) 0
@@ -206,14 +216,10 @@ or do nothing?"
 }
 proc SeditNuke { draft t } {
     global sedit
-    set ix [lsearch $sedit(allids) $sedit($t,toplevel)]
-    if {$ix >= 0} {
-	set sedit(allids) [lreplace $sedit(allids) $ix $ix]
-    }
     catch {destroy .seditUnsent}
     catch {destroy .seditDirty}
-    Exmh_Focus
-    destroy $sedit($t,toplevel)
+    catch {destroy $sedit($t,toplevel).whom}
+    Exwin_Dismiss $sedit($t,toplevel)
 }
 proc SeditMsg { t text } {
     global sedit
@@ -226,7 +232,14 @@ proc SeditMsg { t text } {
 
 proc SeditSend { draft t } {
     if [SeditSave $draft $t] {
+	global env sedit
 	set id [file tail $draft]
+	# Decide if this file needs to go through mhn
+	if {$sedit($t,mhn) && ![catch {exec grep -l ^# $draft}]} {
+	    set env(mhdraft) $draft
+	    SeditMsg $t "Running mhn..."
+	    exec mhn $draft
+	}
 	SeditMsg $t "Message sent"
 	SeditMarkSent $t
 	Edit_Done send $id
@@ -279,7 +292,6 @@ proc SeditSign { draft t {f ~/.signature} } {
 	$t insert end \n
 	$t insert end $signature
 	close $in
-	WidgetTextYview $t end
     }
 }
 proc SeditInsertFile { draft t file } {
@@ -308,36 +320,10 @@ proc SeditInsertFile { draft t file } {
 }
 
 proc SeditInsertFileDialog { draft t } {
-    if [catch {Widget_Toplevel .seditfile "Insert File..."} d] {
-	catch {raise .seditfile}
-	return
+    set name [FSBox "Select file name"]
+    if {$name != ""} {
+	SeditInsertFile $draft $t $name
     }
-    Widget_Message $d msg -aspect 1500 -text "Enter the pathname of the file to insert"
-    set f [Widget_Frame $d name Rim]
-    $f configure -bd 10
-    set f [Widget_Frame $f rim Rim]
-    $f configure -bd 2 -relief raised
-    Widget_Label $f label {left} -text "Pathname:"
-    set e [Widget_Entry $f name {right fill} -bg white -relief raised]
-    bind $e <Return> [list SeditInsertFileCommit .seditfile $e $draft $t]
-    focus $e
-
-    set f [Widget_Frame $d but Rim]
-    $f configure -bd 10
-    Widget_AddBut $f cancel "Cancel" \
-	[list SeditInsertFileDone .seditfile $t] left
-    Widget_AddBut $f ok "OK" \
-	[list SeditInsertFileCommit .seditfile $e $draft $t] right
-}
-proc SeditInsertFileCommit { top entry draft t} {
-    set name [$entry get]
-    SeditInsertFile $draft $t $name
-    SeditInsertFileDone $top $t
-    return
-}
-proc SeditInsertFileDone { top t} {
-    focus $t
-    destroy $top
 }
 
 proc SeditSave { draft t {hook {}} } {
@@ -374,22 +360,38 @@ proc SeditFormatMail { t out } {
     } else {
 	set quote 0
     }
+    set xmailer 0
     set inheaders 1
     scan [$t index end] "%d"  last
+    #
+    # XXX - should nuke all this with on-the-fly line chopping.
+    # Also, should pipe message through mimencode -q if there
+    # is riched text or 8-bit characters.
+    #
     for {set L 1} {$L <= $last} {incr L} {
 	set line [$t get $L.0 $L.end]
 	if {$inheaders} {
+	    # Blank or empty line terminates headers
+	    # Leading --- terminates headers
 	    if {[regexp {^[ 	]*$} $line] || [regexp {^-+} $line]} {
-		# Blank or empty line terminates headers
-		# Leading --- terminates headers
-		if ![info exists exmh(mhaltmsg)] {
+		# No X-Mailer on redistributed messages
+		set id $sedit($t,id)
+		if {![info exists exmh($id,mhannodist)] && !$xmailer} {
 		    puts $out "X-Mailer: exmh $exmh(version)"
 		}
 		SeditMimeHeaders $t $out
 		set inheaders 0
 	    }
+	    if {[regexp {^X-Mailer:} $line]} {
+		set xmailer 1
+	    }
 	}
-	set limit 80
+	if $inheaders {
+	    set limit $sedit(lineLength)
+	} else {
+	    set limit $sedit(lineLength)
+	}
+	set climit [expr $limit-1]
 	set cutoff 50
 	set continuation 0
 	while {[string length $line] > $limit} {
@@ -403,7 +405,11 @@ proc SeditFormatMail { t out } {
 		}
 	    }
 	    if {$c < $cutoff} {
-		set c [expr $limit-1]
+		if {! $inheaders} {
+		    set c [expr $limit-1]
+		} else {
+		    set c [string length $line]
+		}
 	    }
 	    set newline [string range $line 0 $c]
 	    if {! $continuation} {
@@ -411,11 +417,11 @@ proc SeditFormatMail { t out } {
 	    } else {
 		puts $out \ $newline
 	    }
-	   incr c
+	    incr c
 	    set line [string range $line $c end]
 	    if {$inheaders} {
 		set continuation 1
-		set limit 78
+		set limit $climit
 	    }
 	}
 	if {$continuation} {
@@ -442,8 +448,8 @@ proc SeditNotSent { t } {
 
 proc SeditTextBindings { draft t } {
     global sedit
-    bind $t $sedit(key,sendMsg) [list SeditSend $draft $t]
-    bind Entry $sedit(key,sendMsg) { }
+    SeditBind $t sendMsg [list SeditSend $draft $t]
+    SeditBind Entry sendMsg { }
 }
 
 proc SeditSpell { draft f t } {
@@ -507,3 +513,17 @@ proc Sedit_FindMatch { L string } {
     global sedit
     return [FindTextMatch $sedit(searchWidget) $L $string]
 }
+proc Sedit_CheckPoint {} {
+    global sedit
+    foreach top $sedit(allids) {
+	if [info exists sedit($top,id)] {
+	    set draft $sedit($top,draft)
+	    set id $sedit($top,id)
+	    set t $sedit($id,text)
+	    if [SeditIsDirty $t] {
+		Exmh_Status "Saving draft $id"
+		SeditSave $draft $t
+	    }
+	}
+    }
+}
